@@ -892,6 +892,16 @@ im_module_create (const gchar *context_id)
     context_id = &context_id[4];
   }
 
+  uim_init();
+
+  uic->uc =  uim_create_context(uic, "UTF-8",
+				NULL, (char *)context_id,
+				im_uim_commit_string);
+  if (uic->uc == NULL) {
+    parent_class->finalize(obj);
+    return NULL;
+  }
+
   if(im_uim_fd < 0) {
     im_uim_fd     = uim_helper_init_client_fd(im_uim_helper_disconnect_cb);
     if (im_uim_fd >= 0) {
@@ -900,11 +910,6 @@ im_module_create (const gchar *context_id)
     }
   }
 
-  uim_init();
-
-  uic->uc =  uim_create_context(uic, "UTF-8",
-				NULL, (char *)context_id,
-				im_uim_commit_string);
   /**/
   uim_set_mode_cb(uic->uc, mode_cb);
   uim_set_preedit_cb(uic->uc, clear_cb,
@@ -947,7 +952,10 @@ im_module_list(GtkIMContextInfo ***contexts,
   uim_init();
   uc = uim_create_context(NULL, "UTF-8",
 			  NULL, NULL, NULL);
-  nr = uim_get_nr_im(uc);
+  if (uc == NULL)
+    nr = 0;
+  else
+    nr = uim_get_nr_im(uc);
 
   info_list = malloc(sizeof(GtkIMContextInfo*)*nr);
   for (i = 0; i < nr; i++) {
