@@ -121,6 +121,7 @@ button_destroy(gpointer data, gpointer user_data)
 static void
 helper_disconnect_cb(void)
 {
+  uim_fd = -1;
   gdk_input_remove(read_tag);
 }
 
@@ -258,36 +259,26 @@ helper_applet_parse_helper_str(char *str)
 }
 
 static void
-uim_applet_fd_cb(gpointer p, gint fd, GdkInputCondition c)
+uim_applet_fd_read_cb(gpointer p, gint fd, GdkInputCondition c)
 {
-  static char buf[1024];
-  int i,rc;
+  char *tmp;
+  
+  tmp = uim_helper_read_proc(fd);
 
   if(!result)
     result = strdup("");
 
-  while((i = uim_helpler_fd_readable(fd)) > 0) {
-
-    rc = read(fd, buf, sizeof(buf)-1);
-    
-    if(rc == 0) {
-      gdk_input_remove(fd);
-      return;
-    }
-    buf[rc] = '\0';
-    result = (char*)realloc(result, strlen(result) + strlen(buf)+1);
-   
-  strcat(result, buf);
+  if(tmp && tmp[0] != '\0') {
+    result = (char*)realloc(result, strlen(result) + strlen(tmp)+1);
+    strcat(result, tmp);
   }
-  
+
   if(uim_helper_str_terminated(result)) {
     helper_applet_parse_helper_str(result);
     free(result);
     result = NULL;
   }
-  
 
-  free(result);
 }
 
 static GtkWidget * init_toolbar(void)
@@ -315,6 +306,14 @@ prop_button_pressed(GtkButton *prop_button, GdkEventButton *event,
 {
   int x,y;
 
+
+  if(uim_fd < 0) {
+    uim_fd = uim_helper_init_client_fd(helper_disconnect_cb);
+    if(uim_fd > 0)
+      read_tag = gdk_input_add(uim_fd, (GdkInputCondition)GDK_INPUT_READ,
+			       uim_applet_fd_read_cb, 0);    
+  }
+  
   if(!event) {
     return FALSE;
   } else if(event->button == 2 || event->button == 3) {
@@ -363,15 +362,13 @@ uim_applet_new(PanelApplet *applet,
  
   if (strcmp (iid, "OAFIID:GNOME_UimApplet") != 0)
     return FALSE;
-  
-  uim_fd     = uim_helper_init_client_fd(helper_disconnect_cb);
-  
-  toolbar = init_toolbar();
- 
+
+  uim_fd     = uim_helper_init_client_fd(helper_disconnect_cb); 
   if(uim_fd > 0)
     read_tag = gdk_input_add(uim_fd, (GdkInputCondition)GDK_INPUT_READ,
-			     uim_applet_fd_cb, 0);
+			     uim_applet_fd_read_cb, 0);
 
+  toolbar = init_toolbar();
   gtk_container_add (GTK_CONTAINER (applet), toolbar);  
   gtk_widget_show_all (GTK_WIDGET (applet));
 
