@@ -207,6 +207,7 @@
     (prime-context-set-tail! sc '())
     (prime-context-set-left-string! sc '())
     (prime-context-set-right-string! sc '())
+    (prime-context-set-nth! sc nil)
     (prime-context-set-candidate-window! sc #f)))
 
 
@@ -226,49 +227,48 @@
   (lambda (sc sl dir)  ;;dirʸɽ(걦¦ʸϸդˤʤäƤ)
     (let ((kana?))
       (if (= 1 (prime-context-mode sc))
-		  (set! kana? #t))
+	  (set! kana? #t))
       (if (= 2 (prime-context-mode sc))
-		  (set! kana? #f))
+	  (set! kana? #f))
       (if sl
-		  (if dir
-			  (string-append (prime-make-string sc (cdr sl) dir)
-							 (if kana?
-								 (caar sl)
-								 (cdar sl)))
-			  (string-append (if kana?
-								 (caar sl)
-								 (cdar sl))
-							 (prime-make-string sc (cdr sl) dir)))
-	      ""))))
+	  (if dir
+	      (string-append (prime-make-string sc (cdr sl) dir)
+			     (if kana?
+				 (caar sl)
+				 (cdar sl)))
+	      (string-append (if kana?
+				 (caar sl)
+				 (cdar sl))
+			     (prime-make-string sc (cdr sl) dir)))
+	  ""))))
 
 (define prime-context-kana-toggle
   (lambda (sc)
     (let ((s (prime-context-kana-mode sc)))
       (set! s (not s))
       (if s
-		  (prime-context-set-mode! sc 1)
-		  (prime-context-set-mode! sc 2))
+	  (prime-context-set-mode! sc 1)
+	  (prime-context-set-mode! sc 2))
       (prime-context-set-kana-mode! sc s))))
 
 (define prime-get-string-by-mode
   (lambda (sc res)
     (if res
-		(if (prime-context-kana-mode sc)
-			(car res)
-			(cdr res))
-		nil)))
+	(if (prime-context-kana-mode sc)
+	    (car res)
+	    (cdr res))
+	nil)))
 
 (define prime-get-nth-candidate
   (lambda (sc n)
     (if (> n (prime-get-nr-candidates sc))
-		nil)
-	(car (cdr (car (nthcdr n (prime-context-candidates sc)))))
+	nil)
+    (car (cdr (car (nthcdr n (prime-context-candidates sc)))))
     ))
 
 (define prime-get-nr-candidates
   (lambda (sc)
-    (- (length (prime-context-candidates sc)) 1)
-	))
+    (length (prime-context-candidates sc))))
 
 (define prime-get-current-candidate
   (lambda (sc)
@@ -304,12 +304,12 @@
 
 (define prime-learn-word
   (lambda (sc assoc-list)
-	(let ((key     (or (cadr (assoc "basekey"     assoc-list)) ""))
-		  (value   (or (cadr (assoc "base"        assoc-list)) ""))
-		  (part    (or (cadr (assoc "part"        assoc-list)) ""))
-		  (context (or (prime-context-last-word sc) ""))
-		  (suffix  (or (cadr (assoc "conjugation" assoc-list)) ""))
-		  (rest    (or (cadr (assoc "suffix"      assoc-list)) "")))
+    (let ((key     (or (cadr (assoc "basekey"     assoc-list)) ""))
+	  (value   (or (cadr (assoc "base"        assoc-list)) ""))
+	  (part    (or (cadr (assoc "part"        assoc-list)) ""))
+	  (context (or (prime-context-last-word sc) ""))
+	  (suffix  (or (cadr (assoc "conjugation" assoc-list)) ""))
+	  (rest    (or (cadr (assoc "suffix"      assoc-list)) "")))
 
 	  (prime-lib-send-command
 	   (string-append "learn_word\t" key "\t" value "\t" part "\t" context "\t" suffix "\t" rest "\n"))
@@ -317,7 +317,6 @@
 	  (prime-context-set-last-word!
 	   sc
 	   (prime-get-current-candidate sc))
-
 	  )))
 
 (define prime-parse-cands
@@ -335,49 +334,78 @@
 	   (prime-make-string sc (prime-context-left-string sc) #t)
 	   (prime-context-last-word sc))
       (set! res
-			(prime-get-nth-candidate sc 0))
+	    (prime-get-nth-candidate sc 0))
       (if res
-		  (begin
-			(prime-context-set-nth! sc 0)
-			(prime-context-set-state!
-			 sc 'prime-state-converting))
-		  (prime-flush sc))
+	  (begin
+	    (prime-context-set-nth! sc 0)
+	    (prime-context-set-state!
+	     sc 'prime-state-converting))
+	  (prime-flush sc))
       ())))
 
 (define prime-update-preedit
   (lambda (id sc)
     (let ((rkc (prime-context-rk-context sc))
-		  (stat (prime-context-state sc)))
+	  (stat (prime-context-state sc)))
       (im-clear-preedit id)
       (if (= stat 'prime-state-converting)
-		  (begin
-			(im-pushback-preedit
-			 id preedit-reverse
-			 (prime-get-current-candidate sc)))
-		  (if (prime-has-preedit? sc)
-			  (let ((hl (prime-make-string sc
-										   (prime-context-left-string sc)
-										   #t))
-					(hr (prime-make-string sc
-										   (prime-context-right-string sc)
-										   #f)))
-				(if (string? hl)
-					(im-pushback-preedit
-					 id preedit-underline
-					 hl))
-				(im-pushback-preedit id preedit-underline
-									 (rk-pending rkc))
-				(im-pushback-preedit id preedit-cursor "")
-				(if (string? hr)
-					(im-pushback-preedit
-					 id preedit-underline
-					 hr)))))
+	  (begin
+	    (im-pushback-preedit
+	     id preedit-reverse
+	     (prime-get-current-candidate sc)))
+	  (if (prime-has-preedit? sc)
+	      (let ((hl (prime-make-string
+			 sc (prime-context-left-string sc) #t))
+		    (hr (prime-make-string
+			 sc (prime-context-right-string sc) #f)))
+		(if (string? hl)
+		    (im-pushback-preedit
+		     id preedit-underline
+		     hl))
+		(im-pushback-preedit id preedit-underline
+				     (rk-pending rkc))
+		(im-pushback-preedit id preedit-cursor "")
+		(if (string? hr)
+		    (im-pushback-preedit
+		     id preedit-underline
+		     hr)))))
       (im-update-preedit id))))
+
 (define prime-update-mode
   (lambda (id sc)
     (let ((mode (prime-context-mode sc)))
       (im-update-mode id mode))))
 
+(define prime-update-candidate-window
+  (lambda (sc id)
+    (if (and
+	 (prime-has-preedit? sc)
+	 (not (prime-context-candidate-window sc))
+	 (or prime-always-show-window?
+	     (> (prime-context-candidate-op-count sc)
+		prime-candidate-op-count)))
+	(begin
+	  (prime-get-candidates!
+	   sc
+	   (prime-make-string sc (prime-context-left-string sc) #t)
+	   (prime-context-last-word sc))
+	  (im-begin-candidate
+	   id (prime-get-nr-candidates sc) 0)
+	  (prime-context-set-candidate-window! sc #t)
+	  ))
+    (if (not (prime-has-preedit? sc))
+	(begin
+	  (prime-context-set-candidate-window! sc #f)
+	  (im-end-candidate id)))
+    (if (prime-context-candidate-window sc)
+	(begin
+	  (prime-get-candidates!
+	   sc
+	   (prime-make-string sc (prime-context-left-string sc) #t)
+	   (prime-context-last-word sc))	  
+	  (im-update-candidate id (prime-context-nth sc))))
+    ))
+
 (define prime-has-preedit?
   (lambda (ac)
     (or
@@ -388,364 +416,292 @@
 (define prime-proc-input-no-preedit
   (lambda (c key key-state)
     (let* ((sc (context-data c))
-		   (id (context-id c))
-		   (key-str (charcode->string (to-lower-char key)))
-		   (rkc (prime-context-rk-context sc))
-		   (res)
-		   (immediate-commit))
-      (set! res nil)
+	   (id (context-id c))
+	   (key-str (charcode->string (to-lower-char key)))
+	   (rkc (prime-context-rk-context sc))
+	   (res nil)
+	   (direct (ja-direct (charcode->string key)))
+	   (immediate-commit))
       (and
        (if (prime-wide-latin-key key key-state)
-		   (begin
-			 (prime-context-set-mode! sc 3)
-			 (prime-update-mode id sc)
-			 #f)
-		   #t)
+	   (begin
+	     (prime-context-set-mode! sc 3)
+	     (prime-update-mode id sc)
+	     #f)
+	   #t)
        (if (prime-latin-key key key-state)
-		   (begin
-			 (prime-context-set-mode! sc 0)
-			 (prime-update-mode id sc)
-			 #f)
-		   #t)
+	   (begin
+	     (prime-context-set-mode! sc 0)
+	     (prime-update-mode id sc)
+	     #f)
+	   #t)
        (if (prime-cancel-key key key-state)
-		   (begin
-			 (prime-flush sc)
-			 #f)
-		   #t)
+	   (begin
+	     (prime-flush sc)
+	     #f)
+	   #t)
        (if (prime-kana-toggle key key-state)
-		   (begin 
-			 (prime-context-kana-toggle sc)
-			 (prime-update-mode id sc)
-			 #f)
-		   #t)
+	   (begin 
+	     (prime-context-kana-toggle sc)
+	     (prime-update-mode id sc)
+	     #f)
+	   #t)
        (if (prime-backspace-key key key-state)
-		   (if (not (rk-backspace rkc))
-			   (begin
-				 (im-commit-raw (context-id c))
-				 #f)
-			   #f)
-		   #t)
-       (if (or
-			(control-key-mask key-state)
-			(= key 32))
-		   (begin
-			 (im-commit-raw id)
-			 #f)
-		   #t)
+	   (if (not (rk-backspace rkc))
+	       (begin
+		 (im-commit-raw (context-id c))
+		 #f)
+	       #f)
+	   #t)
+       (if (control-key-mask key-state)
+	   (begin
+	     (im-commit-raw id)
+	     #f)
+	   #t)
        (if (and
-			(shift-key-mask key-state)
-			(alphabet-char? key))
-		   (begin
-			 (prime-context-set-immediate-commit! sc #f)
-			 #t)
-		   #t)
-       (if (and
-			(not (prime-context-left-string sc))
-			(not (string-find (rk-expect rkc) key-str))
-			(not (rk-pending rkc)))
-		   (begin
-			 (im-commit-raw id)
-			 (prime-flush sc)
-			 (prime-update-preedit id sc)
-			 #f)
-		   #t)
+	    (shift-key-mask key-state)
+	    (alphabet-char? key))
+	   (begin
+	     (prime-context-set-immediate-commit! sc #f)
+	     #t)
+	   #t)
+       ;; direct key => commit
+       (if direct
+	   (begin
+	     (im-commit id direct)
+	     #f)
+	   #t)
        (if (symbol? key)
-		   (begin
-			 (prime-flush sc)
-			 (prime-context-set-last-word ! sc "")
-			 (im-commit-raw (context-id c))
-			 #f)
-		   #t)
-       (begin
-		 (set! res
-			   (rk-push-key!
-				rkc
-				key-str))
-		 #t))
-	  
-      ;; update state
-      (if res (begin
-				(prime-context-set-left-string! sc
-												(cons
-												 res
-												 (prime-context-left-string sc)))
-				(if (and prime-preedit-immididate-commit? (prime-context-immediate-commit sc))
-					(begin 
-					  (im-commit id (prime-make-string sc (prime-context-left-string sc) #t))
-					  (prime-flush sc)
-					  #f)
-					#t)
-				(prime-get-candidates!
-				 sc
-				 (prime-make-string sc (prime-context-left-string sc) #t)
-				 (prime-context-last-word sc))
-				(prime-context-set-state! sc 'prime-state-with-preedit)))
-	  nil)))
+	   (begin
+	     (prime-flush sc)
+	     (prime-context-set-last-word! sc "")
+	     (im-commit-raw (context-id c))
+	     #f)
+	   #t)
+       (prime-proc-input-with-preedit c key key-state))
+      nil)))
 
 (define prime-proc-input-with-preedit
   (lambda (c key key-state)
     (let* ((sc (context-data c))
-		   (id (context-id c))
-		   (rkc (prime-context-rk-context sc))
-		   (stat (prime-context-state sc))
-		   (res))
+	   (id (context-id c))
+	   (rkc (prime-context-rk-context sc))
+	   (stat (prime-context-state sc))
+	   (res))
       (and
        (if (prime-begin-conv-key key key-state)
-		   (begin
-			 (prime-begin-conversion sc id)
-			 #f)
-		   #t)
+	   (begin
+	     (prime-begin-conversion sc id)
+	     #f)
+	   #t)
        (if (prime-cancel-key key key-state)
-		   (begin
-			 (prime-flush sc)
-			 #f)
-		   #t)
+	   (begin
+	     (prime-flush sc)
+	     #f)
+	   #t)
        (if (prime-backspace-key key key-state)
-		   (begin
-			 (if (not (rk-backspace rkc))
-				 (if (prime-has-preedit? sc)
-					 (prime-context-set-left-string!
-					  sc (cdr (prime-context-left-string sc)))
-					 (begin
-					   (im-commit-raw id)
-					   (prime-flush sc))))
-			 #f)
-		   #t)
-	   ;; delete
+	   (begin
+	     (if (not (rk-backspace rkc))
+		 (if (prime-has-preedit? sc)
+		     (prime-context-set-left-string!
+		      sc (cdr (prime-context-left-string sc)))
+		     (begin
+		       (im-commit-raw id)
+		       (prime-flush sc))))
+	     #f)
+	   #t)
+       ;; delete
        (if (prime-delete-key key key-state)
-		   (begin
-			 (if (not (rk-delete rkc))
-				 (if (prime-context-right-string sc)
-					 (prime-context-set-right-string!
-					  sc
-					  (cdr (prime-context-right-string sc)))))
-			 #f)
-		   #t)
-       ;; cancel
-       (if (prime-cancel-key key key-state)
-		   (begin
-			 (prime-flush sc)
-			 #f)
-		   #t)
+	   (begin
+	     (if (not (rk-delete rkc))
+		 (if (prime-context-right-string sc)
+		     (prime-context-set-right-string!
+		      sc
+		      (cdr (prime-context-right-string sc)))))
+	     #f)
+	   #t)
        ;;
        (if (prime-commit-key key key-state)
-		   (begin
-			 (im-commit id  (prime-make-string sc (prime-context-left-string sc) (prime-context-kana-mode sc)))
-			 (prime-flush sc)
-			 (prime-update-mode id sc)
-			 #f)
-		   #t)
-	   ;; left
+	   (begin
+	     (im-commit id (prime-make-string
+			    sc (prime-context-left-string sc) (prime-context-kana-mode sc)))
+	     (prime-flush sc)
+	     (prime-update-mode id sc)
+	     #f)
+	   #t)
+       ;; left
        (if (prime-go-left-key key key-state)
-		   (begin
-			 (if (prime-context-left-string sc)
-				 (let ((c (car (prime-context-left-string sc))))
-				   (prime-context-set-left-string!
-					sc (cdr (prime-context-left-string sc)))
-				   (prime-context-set-right-string! sc
-													(cons c
-														  (prime-context-right-string sc)))))
-			 #f)
-		   #t)
+	   (begin
+	     (if (prime-context-left-string sc)
+		 (let ((c (car (prime-context-left-string sc))))
+		   (prime-context-set-left-string!
+		    sc (cdr (prime-context-left-string sc)))
+		   (prime-context-set-right-string! 
+		    sc
+		    (cons c (prime-context-right-string sc)))))
+	     #f)
+	   #t)
        ;; right
        (if (prime-go-right-key key key-state)
-		   (begin
-			 (if (prime-context-right-string sc)
-				 (let ((c (car (prime-context-right-string sc))))
-				   (prime-context-set-right-string!
-					sc (cdr (prime-context-right-string sc)))
-				   (prime-context-set-left-string! sc
-												   (cons c
-														 (prime-context-left-string sc)))))
-			 #f)
-		   #t)
-	   ;; modifiers (shift) => ignore
+	   (begin
+	     (if (prime-context-right-string sc)
+		 (let ((c (car (prime-context-right-string sc))))
+		   (prime-context-set-right-string!
+		    sc (cdr (prime-context-right-string sc)))
+		   (prime-context-set-left-string!
+		    sc
+		    (cons c (prime-context-left-string sc)))))
+	     #f)
+	   #t)
+       (if (numeral? key)
+	   (begin
+	     (prime-context-set-nth! sc (- key 49))
+	     (im-commit id (prime-get-nth-candidate sc (- key 49)))
+	     (prime-commit-candidate sc)
+	     (prime-flush sc)
+	     (prime-update-mode id sc)
+	     #f)
+	   #t)
+       ;; modifiers (shift) => ignore
        (if (and (modifier-key-mask key-state) (not (shift-key-mask key-state)))
-		   (begin
-			 (im-commit-raw id)
-			 #f)
-		   #t)
+	   (begin
+	     (im-commit-raw id)
+	     #f)
+	   #t)
        (begin
-		 (set! res
-			   (rk-push-key!
-				rkc
-				(charcode->string (to-lower-char key))))
-		 (if res
-			 (begin
-			   (prime-context-set-left-string! sc
-											   (cons
-												res
-												(prime-context-left-string sc)))
-			   (if (and prime-preedit-immididate-commit? (prime-context-immediate-commit sc))
-				   (begin 
-					 (im-commit id (prime-make-string sc (prime-context-left-string sc) #t))
-					 (prime-flush sc)
-					 #f)
-				   #t)
-			   ))
-		 ))
+	 (set! res
+	       (rk-push-key!
+		rkc
+		(charcode->string (to-lower-char key))))
+	 (if res
+	     (begin
+	       (prime-context-set-left-string!
+		sc
+		(cons res
+		      (prime-context-left-string sc)))
+	       (prime-context-set-candidate-window! sc #f) ;FIXME:very duty hack
+	       (if (and prime-preedit-immididate-commit? (prime-context-immediate-commit sc))
+		   (begin 
+		     (im-commit id (prime-make-string sc (prime-context-left-string sc) #t))
+		     (prime-flush sc)
+		     #f)
+		   #t)
+	       ))))
       nil)))
 
 (define prime-proc-state-converting
   (lambda (c key key-state)
     (let ((sc (context-data c))
-		  (id (context-id c))
-		  (res ()))
+	  (id (context-id c))
+	  (res ()))
       (and
        (if (prime-next-candidate key key-state)
-		   (begin
-			 (prime-context-set-nth! sc
-									 (+ 1 (prime-context-nth sc)))
-			 (prime-context-set-candidate-op-count! sc
-													(+ 1 (prime-context-candidate-op-count sc)))
-			 (if (not (prime-get-current-candidate sc))
-				 (prime-context-set-nth! sc 0))
-			 ;; Windowɽ򳫻Ϥ뤫
-			 (if
-			  (and
-			   (not
-				(prime-context-candidate-window sc))
-			   prime-use-candidate-window?
-			   (> (prime-context-candidate-op-count sc)
-				  prime-candidate-op-count))
-			  (begin
-				(prime-context-set-candidate-window! sc #t)
-				(im-begin-candidate
-				 id
-				 (prime-get-nr-candidates sc)
-				 0)))
-			 ;;
-			 (if (prime-context-candidate-window sc)
-				 (im-update-candidate id (prime-context-nth sc)))
-			 #f)
-		   #t)
+	   (begin
+	     (prime-context-set-nth! sc
+				     (+ 1 (prime-context-nth sc)))
+	     (if (not (prime-get-current-candidate
+		       sc (prime-context-nth sc)))
+	     (prime-context-set-nth! sc 0))
+	     (prime-context-set-candidate-op-count!
+	      sc
+	      (+ 1 (prime-context-candidate-op-count sc)))
+	     #f)
+	   #t)
        (if (prime-prev-candidate key key-state)
-		   (begin
-			 (if (> (prime-context-nth sc) 0)
-				 (prime-context-set-nth! sc (- (prime-context-nth sc) 1))
-				 (prime-context-set-nth! sc (- (prime-get-nr-candidates sc) 1)))
-			 (if
-			  (and
-			   (not
-				(prime-context-candidate-window sc))
-			   prime-use-candidate-window?
-			   (> (prime-context-candidate-op-count sc)
-				  prime-candidate-op-count))
-			  (begin
-				(prime-context-set-candidate-window! sc #t)
-				(im-begin-candidate
-				 id
-				 (prime-get-nr-candidates sc)
-				 0)))
-			 (if (prime-context-candidate-window sc)
-				 (im-update-candidate id (prime-context-nth sc)))
-			 #f)
-		   #t)
+	   (begin
+	     (if (> (prime-context-nth sc) 0)
+		 (prime-context-set-nth! sc (- (prime-context-nth sc) 1))
+		 (prime-context-set-nth! sc (- (prime-get-nr-candidates sc) 1)))
+	     #f)
+	   #t)
        (if (prime-cancel-key key key-state)
-		   (begin
-			 (if (prime-context-candidate-window sc)
-				 (im-end-candidate id))
-			 (prime-flush sc)
-			 #f)
-		   #t)
+	   (begin
+	     (prime-flush sc)
+	     #f)
+	   #t)
        (if (prime-commit-key key key-state)
-		   (begin
-			 (set! res (prime-get-current-candidate sc))
-			 (set!
-			  res
-			  (string-append res
-							 (prime-make-string sc (prime-context-tail sc)
-												(prime-context-kana-mode sc))))
-			 (prime-commit-candidate sc)
-			 (if (prime-context-candidate-window sc)
-				 (im-end-candidate id))
-			 (prime-flush sc)
-			 (prime-context-set-state! sc 'prime-state-no-preedit)
-			 (prime-update-mode id sc)
-			 #f)
-		   #t)
+	   (begin
+	     (set! res (prime-get-current-candidate sc))
+	     (prime-commit-candidate sc)
+	     (prime-flush sc)
+	     (prime-update-mode id sc)
+	     #f)
+	   #t)
        (begin
-		 (prime-context-set-state! sc 'prime-state-no-preedit)
-		 (prime-update-mode id sc)
-		 (set! res (prime-get-current-candidate sc))
-		 (if (prime-context-candidate-window sc)
-			 (im-end-candidate id))
-		 (set!
-		  res
-		  (string-append res 
-						 (prime-make-string sc
-											(prime-context-tail sc)
-											(prime-context-kana-mode sc))))
-		 (prime-commit-candidate sc)
-		 (prime-flush sc)
-		 (let ((res2 (prime-proc-input-no-preedit c key key-state)))
-		   (set!
-			res
-			(string-append res 
-						   (prime-make-string sc
-											  (prime-context-tail sc)
-											  (prime-context-kana-mode sc))))
-		   (if (string? res2)
-			   (set! res
-					 (string-append res res2))))))
-	  res)))
+	 (prime-update-mode id sc)
+	 (set! res (prime-get-current-candidate sc))
+	 (prime-commit-candidate sc)
+	 (prime-flush sc)
+	 (let ((res2 (prime-proc-input-no-preedit c key key-state)))
+	   (set!
+	    res
+	    (string-append
+	     res 
+	     (prime-make-string sc
+				(prime-context-tail sc)
+				(prime-context-kana-mode sc))))
+	   (if (string? res2)
+	       (set! res
+		     (string-append res res2))))))
+      res)))
 
 (define prime-proc-mode-latin
   (lambda (c key key-state)
     (let ((sc (context-data c))
-		  (id (context-id c)))
+	  (id (context-id c)))
       (if
        (prime-on-key key key-state)
        (begin
-		 (prime-context-set-mode! sc 1)
-		 (prime-update-mode id sc))
+	 (prime-context-set-mode! sc 1)
+	 (prime-update-mode id sc))
        (im-commit-raw id))
       ())))
 
 (define prime-proc-mode-wide-latin
   (lambda (c key key-state)
     (let* ((w (ja-wide (charcode->string key)))
-		   (id (context-id c))
-		   (sc (context-data c)))
+	   (id (context-id c))
+	   (sc (context-data c)))
       (if (prime-on-key key key-state)
-		  (begin
-			(prime-flush sc)
-			(prime-context-set-mode! sc 1)
-			(prime-update-mode id sc))
-		  (if w
-			  (im-commit id w)
-			  (im-commit-raw id)))
+	  (begin
+	    (prime-flush sc)
+	    (prime-context-set-mode! sc 1)
+	    (prime-update-mode id sc))
+	  (if w
+	      (im-commit id w)
+	      (im-commit-raw id)))
       ())))
 
 (define prime-push-key
   (lambda (c key key-state)
     (let* ((sc (context-data c))
-		   (state (prime-context-state sc))
-		   (mode (prime-context-mode sc))
-		   (fun)
-		   (res))
+	   (state (prime-context-state sc))
+	   (mode (prime-context-mode sc))
+	   (fun)
+	   (res))
       (if (= mode 0)
-		  (set! fun prime-proc-mode-latin))
+	  (set! fun prime-proc-mode-latin))
       (if (= mode 3)
-		  (set! fun prime-proc-mode-wide-latin))
+	  (set! fun prime-proc-mode-wide-latin))
       (if (or (= mode 1) (= mode 2))
-		  (begin
-			(if (prime-has-preedit? sc)
-				(set! fun prime-proc-input-with-preedit)
-				(set! fun prime-proc-input-no-preedit))
-			(if (= state 'prime-state-converting)
-				(set! fun prime-proc-state-converting))))
-      (set! res (fun c key key-state))      
+	  (begin
+	    (if (prime-has-preedit? sc)
+		(set! fun prime-proc-input-with-preedit)
+		(set! fun prime-proc-input-no-preedit))
+	    (if (= state 'prime-state-converting)
+		(set! fun prime-proc-state-converting))))
+      (set! res (fun c key key-state))  
       (if res
-		  (im-commit (context-id c) res))
-      (prime-update-preedit
-       (context-id c) sc))))
+	  (im-commit (context-id c) res))
+      (prime-update-preedit (context-id c) sc)
+      (prime-update-candidate-window sc (context-id c))
+      )))
 
 (define prime-init-handler
   (lambda (id arg)
     (let* ((c (find-context id)))
       (set-context-data! c
-						 (prime-context-new))
+			 (prime-context-new))
       (im-clear-mode-list id)
       (im-pushback-mode-list id "RAW")
       (im-pushback-mode-list id "Ҥ餬")
@@ -767,30 +723,29 @@
   (lambda (id)
     ()))
 
-
 (define prime-mode-handler
   (lambda (id mode)
     (let* ((c (find-context id))
-		   (sc (context-data c)))
-	  (prime-flush sc)
-	  (prime-context-set-mode! sc mode)
-	  (if (= mode 1)
-		  (prime-context-set-kana-mode! sc #t))
-	  (if (= mode 2)
-		  (prime-context-set-kana-mode! sc #f))
-	  (prime-update-preedit id sc)
-	  ())))
+	   (sc (context-data c)))
+      (prime-flush sc)
+      (prime-context-set-mode! sc mode)
+      (if (= mode 1)
+	  (prime-context-set-kana-mode! sc #t))
+      (if (= mode 2)
+	  (prime-context-set-kana-mode! sc #f))
+      (prime-update-preedit id sc)
+      ())))
 
 (define prime-get-candidate-handler
   (lambda (id idx)
     (let* ((c (find-context id))
-		   (sc (context-data c)))
+	   (sc (context-data c)))
       (prime-get-nth-candidate sc idx))))
 
 (define prime-set-candidate-index-handler
   (lambda (id idx)
     (let* ((c (find-context id))
-		   (sc (context-data c)))
+	   (sc (context-data c)))
       (prime-context-set-nth! sc idx)
       (prime-update-preedit  id sc))))
 
