@@ -25,6 +25,8 @@
 (define skk-use-candidate-window? #t)
 (define skk-candidate-op-count 0)
 (define skk-use-recursive-learning? #f)
+; 47 = '/'
+(define skk-latin-conv-key 47)
 ;; key defs
 (define skk-latin-key
   (lambda (key key-state)
@@ -68,6 +70,14 @@
     (generic-backspace-key key key-state)))
 
 ;; access
+(define skk-context-latin-conv
+  (lambda (c)
+    (car (nthcdr 13 c))))
+
+(define skk-context-set-latin-conv!
+  (lambda (c lc)
+    (set-car! (nthcdr 13 c) lc)))
+
 (define skk-context-editor
   (lambda (c)
     (car (nthcdr 12 c))))
@@ -203,7 +213,8 @@
     (skk-context-set-okuri-head! sc "")
     (skk-context-set-okuri! sc '())
     (skk-context-set-candidate-op-count! sc 0)
-    (skk-context-set-candidate-window! sc #f)))
+    (skk-context-set-candidate-window! sc #f)
+    (skk-context-set-latin-conv! sc #f)))
 
 (define skk-context-new
   (lambda (id)
@@ -216,7 +227,7 @@
 	  (skk-read-personal-dictionary)))
     (let ((sc 
 	   (copy-list 
-	    '(skk-state-latin #t "" "" "" () () () () () () () () ()))))
+	    '(skk-state-latin #t "" "" "" () () () () () () () () () ()))))
       (skk-context-set-head! sc ())
       (skk-context-set-rk-context! sc
 				   (rk-context-new ja-rk-rule #t #f))
@@ -444,6 +455,13 @@
 	     (skk-commit-raw sc id key key-state)
 	     #f)
 	   #t)
+       (if (= key skk-latin-conv-key)
+	   (begin
+	     (skk-context-set-state! sc 'skk-state-kanji)
+	     (skk-update-mode id sc)
+	     (skk-context-set-latin-conv! sc #t)
+	     #f)
+	   #t)
        (if (and
 	    (shift-key-mask key-state)
 	    (alphabet-char? key))
@@ -475,7 +493,7 @@
 	       (rk-push-key!
 		rkc
 		key-str))
-	 #t))
+	 #t));;and
       ;; update state
       (if (= (skk-context-state sc) 'skk-state-kanji)
 	  (if res
