@@ -35,6 +35,55 @@ uim_ipc_open_command(int old_pid, FILE **read_handler, FILE **write_handler, cha
   return 0;
 }
 
+
+
+char *
+uim_ipc_send_command(int *pid, FILE *read_handler, FILE *write_handler, char* command, char *str)
+{
+  char *tmp = strdup("");
+  char buf[8192];
+
+  if (read_handler == NULL || write_handler == NULL)
+    *pid = uim_ipc_open_command(*pid, &read_handler, &write_handler, command );
+  
+  if(*pid == 0) {
+    free(tmp);
+    return NULL;
+  }
+
+  fprintf( write_handler, str );
+  
+
+ again:
+  if (fflush(write_handler) != 0) {
+    switch (errno) {
+    case EINTR:
+      goto again;
+    default:
+      goto err;
+    }
+  }
+
+  while (fgets (buf, sizeof(buf), read_handler ) != NULL ) {
+    if(strcmp( buf, "\n" ) == 0)
+      break;
+
+    tmp = realloc(tmp, strlen(tmp) + strlen(buf) + 1);
+    strcat(tmp, buf);
+  }
+ 
+  if (strlen(tmp) == 0)
+    goto err;
+
+  return tmp;
+
+ err:
+  free(tmp);
+  *pid = uim_ipc_open_command(*pid, read_handler, write_handler, command );
+  return NULL;
+}
+
+
 static pid_t
 open_pipe_rw(FILE **fr, FILE **fw)
 {
