@@ -105,7 +105,7 @@ str_seq_partial(LISP seq, LISP rule)
   return sym_t;
 }
 
-LISP
+static LISP
 rk_find_seq(LISP s, LISP rule)
 {
   for (; rule != NIL; rule = CDR(rule)) {
@@ -116,7 +116,7 @@ rk_find_seq(LISP s, LISP rule)
   return NIL;
 }
 
-LISP
+static LISP
 rk_find_partial_seq(LISP s, LISP rule)
 {
   for (; rule != NIL; rule = CDR(rule)) {
@@ -166,8 +166,8 @@ uim_init_util_subrs()
   init_subr_1("charcode->string", charcode2string);
   init_subr_2("str-seq-equal?", str_seq_equal);
   init_subr_2("str-seq-partial?", str_seq_partial);
-  init_subr_2("rk-find-seq", rk_find_seq);
-  init_subr_2("rk-find-partial-seq", rk_find_partial_seq);
-  init_subr_2("rk-expect-seq", rk_expect_seq);
+  init_subr_2("rk-lib-find-seq", rk_find_seq);
+  init_subr_2("rk-lib-find-partial-seq", rk_find_partial_seq);
+  init_subr_2("rk-lib-expect-seq", rk_expect_seq);
   init_subr_1("getenv", c_getenv);
 }
