@@ -323,16 +323,23 @@ load_conf()
   struct passwd *pw;
   char *hd;
   FILE *fp;
+  long svl = siod_verbose_level;
+  long ret;
+
   pw = getpwuid(getuid());
-  hd = alloca(strlen(pw->pw_dir)+10);
+  hd = alloca(strlen(pw->pw_dir)+40);
   sprintf(hd, "%s/.uim", pw->pw_dir);
   fp = fopen(hd, "r");
-  if (fp) {
-    fclose(fp);
-    vload(hd, 0, 1);
-    return 0;
-  }
-  return -1;
+  if (fp == NULL)
+    return -1;
+  fclose(fp);
+
+  if (siod_verbose_level < 1)
+    siod_verbose_level = 1;	/* to show error message of SIOD */
+  sprintf(hd, "(*catch 'errobj (load \"%s/.uim\" #f #f))", pw->pw_dir);
+  ret = repl_c_string(hd, 0, 0, 0);
+  siod_verbose_level = svl;
+  return ret;
 }
 
 int uim_set_candidate_cb(uim_context uc,
@@ -393,9 +400,20 @@ uim_init_scm()
       "-n2048",
       "-v0",
     };
+  char verbose_argv[4] = "-v4";
+  char *env;
 
+  if ((env = getenv("LIBUIM_VERBOSE"))) {
+    if (isdigit(*env)) {
+      if (isdigit(*(env+1)))
+	verbose_argv[2] = '9';	/* SIOD's max verbose level is 5 */
+      else
+	verbose_argv[2] = *env;
+    }
+    siod_argv[6] = verbose_argv;
+  }
   /* init siod */
-  siod_output = stdout;
+  siod_output = stderr;
   process_cla (7, siod_argv, 1);
   init_storage ();
   init_subrs ();
